//! VImageCleaning  image cleaning routines

#ifndef VIMAGECLEANING_H
#define VIMAGECLEANING_H

#include <VEvndispData.h>
#include <TGraphErrors.h>

#include "VImageCleaningRunParameter.h"

using namespace std;

class VImageCleaning
{
	private:
	
		VEvndispData* fData;
                bool fWriteGraphToFileRecreate;
		
		void cleanImageWithTiming( VImageCleaningRunParameter *iImageCleaningParameters, bool isFixed );
		void fillImageBorderNeighbours();
		void fillBorderBorderNeighbours();
		void recoverImagePixelNearDeadPixel();
		void printDataError( string iFunctionName );
                
		// cluster cleaning
		void mergeClusters();
		void removeSmallClusters( int );
		void addToCluster( unsigned int cID, unsigned int iChan);
		void removeCluster( unsigned int cID) ;
		vector<int> fNpixCluster;
		vector<double> fSizeCluster;

		// NN image cleaning
		bool  kInitNNImageCleaning;
		bool  kInitNNImgClnPerTelType[VDST_MAXTELTYPES];
                unsigned int fIPRdim;
		TObjArray* fProb4nnCurves;
		TObjArray* fProb3nnrelCurves;
		TObjArray* fProb2plus1Curves;
		TObjArray* fProb2nnCurves;
		TObjArray* fProbBoundCurves;
		TObjArray* fIPRgraphs;
                bool ifActiveNN[VDST_MAXNNGROUPTYPES][VDST_MAXTELTYPES];   // if  NN groups is searched in NN-image cleaning procedure

		int   VALIDITY[VDST_MAXCHANNELS];      //   Flags for pixels, accepted by nn-image cleaning. VALIDITY[i]=2-6 : core pixels, VALIDITY[i]>6 :boundary pixels
		int   VALIDITYBOUND[VDST_MAXCHANNELS]; //
		int   VALIDITYBUF[VDST_MAXCHANNELS];   //
		unsigned int nRings;
		float CoincWinLimit;                   //ns
		double fNSBscale;
		double fSPhePulseFWHM;
		double fIntegWindow;  // ns
		float fFADCtoPhe[VDST_MAXTELTYPES];
		float fMinRate[VDST_MAXTELTYPES];      //
		
		float INTENSITY[VDST_MAXCHANNELS];     //
		float TIMES[VDST_MAXCHANNELS];         //
		float** IPR;                           // IPR[TelType][ScanDim] scan. Not used TelType==0 is filled with DT values

		int   LocMin( int n, float* ptr, float& min );
		int   LocMax( int n, float* ptr, float& max );
		
		// main functions
		bool  BoundarySearch( int TrigSimTelType, float thresh, TF1* fProbCurve, float refdT, int refvalidity, int idx );
		int   NNGroupSearchProbCurve( int TrigSimTelType, TF1* fProbCurve, float PreCut );
		int   NNGroupSearchProbCurveRelaxed( int TrigSimTelType, TF1* fProbCurve, float PreCut );
		void  ScaleCombFactors( int TrigSimTelType, float scale );
		void  ResetCombFactors( int TrigSimTelType );
		float ImageCleaningCharge( int TrigSimTelType, int& ngroups );
		bool  InitNNImageCleaning();
		bool  InitNNImgClnPerTelType( int TrigSimTelType);
		int   getTrigSimTelType( unsigned int fTelType );
		void  DiscardTimeOutlayers( int TrigSimTelType );
		void  DiscardLocalTimeOutlayers( int TrigSimTelType, float NNthresh[6] ); // use this function
		void  DiscardIsolatedPixels( int TrigSimTelType );
		void  FillIPR( unsigned int TrigSimTelType);
		void  FillPreThresholds( TGraph* gipr, float NNthresh[6] ); // defines pre-search thresholds for nn-groups (below this threshold group is not searched)
		TGraphErrors* GetIPRGraph( unsigned int TrigSimTelType, float ScanWidow );
		void  SetNeighborRings( int TrigSimTelType, unsigned short* VALIDITYBOUNDBUF, float* TIMESReSearch, float* REFTHRESH );
                

                TF1* defineRateContourFunction( int type, TString funcname, float iRate, float iNfold, float iCombFactor,
                                                float xlow, float xup );
                TF1* defineRateContourBoundFunction( int type, TString funcname, float iRate, float iNfold, float iCombFactor,
                                                     float xlow, float xup );
                void writeProbabilityCurve( TGraph *iIPR, TF1 *iProb, double iRate );
		
	public:
	
		VImageCleaning( VEvndispData* iData = 0 );
	       ~VImageCleaning() {}
		
		// tailcut cleaning
		void cleanImageFixed( VImageCleaningRunParameter *iImageCleaningParameters );
                void cleanImageFixed( double hithresh, double lothresh, double brightthresh = -999. );
		void cleanImagePedvars( VImageCleaningRunParameter *iImageCleaningParameters );
		
		// time tailcut cleaning
		void cleanImagePedvarsTimeDiff( VImageCleaningRunParameter *iImageCleaningParameters );
		
		// time cluster cleaning
		void cleanImageFixedWithTiming( VImageCleaningRunParameter *iImageCleaningParameters );
		void cleanImagePedvarsWithTiming( VImageCleaningRunParameter *iImageCleaningParameters );

		// cluster cleaning
		void cleanImageWithClusters( VImageCleaningRunParameter *iImageCleaningParameters, bool isFixed );

                // produce a trigger map
		void cleanTriggerFixed( VImageCleaningRunParameter *iImageCleaningParameters );
                
		// trace correlation cleaning 
		void cleanImageTraceCorrelate( VImageCleaningRunParameter *iImageCleaningParameters );

                // NN image cleaning
		void  cleanNNImageFixed();
			
		
		void addImageChannel( unsigned int iChannel );                      // add this pixel to image
		void removeImageChannel( unsigned int iChannel );                   // remove this pixel from image
		void resetImageChannel( unsigned int iChannel );                    // reset this pixel to standard value
		
};
#endif
