/*   
    small macro which reads a prod3 list file from KB and writes array layout files

    (e.g. Prod3-Layouts-new2.lis)

*/

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

using namespace std;

void writeProd3_array( string iKB_file, string iArrayLayout, string iMST, string iSST, bool iSouth )
{
	///////////////////////////////////////////////
	// input:
	// open KB-style file with array layouts
	ifstream is;
	is.open( iKB_file.c_str(), ifstream::in );
	if( !is )
	{
		cout << "error, KB-style input file not found: " << endl;
		cout << iKB_file << endl;
		return;
	}
	///////////////////////////////////////////////
	// input:
	// prod3 lis file South:
	string iOFile;
        if( iSouth ) 
        {
             iOFile = "CTA.prod3S." + iArrayLayout + "-" + iMST + iSST + ".lis";
        }
        else
        {
             iOFile = "CTA.prod3N." + iArrayLayout + "-" + iMST + ".lis";
        }
	cout << iOFile << endl;
	ofstream os;
	os.open( iOFile.c_str() );
	if( !os )
	{
	      cout << "error opening output file: " << iOFile << endl;
	      return;
        }
	cout << "Writing results to " << iOFile << endl;
	
	string is_line;
	string iTemp;
	string iTemp2;
	while( getline( is, is_line ) )
	{
		// empty line
		if( is_line.size() == 0 )
		{
		     continue;
                }
	        istringstream is_stream( is_line );
	        is_stream >> iTemp;
	        // '*' is a comment
	        if( iTemp == "*" )
	        {
		   continue;
	        }

	        // new array layout
	        if( iTemp == "Layout" )
	        {
		    is_stream >> iTemp;
		    if( iTemp.size() > 0 )
		    {
		       iTemp = iTemp.substr( 0, iTemp.size()-1 );
		       if( iTemp == iArrayLayout )
		       {
			    /////////////////////////////////////////
			    cout << "FOUND LAYOUT " << iTemp << endl;
                            bool iFoundLSTs = false;
                            bool iFoundMSts = false;
                            bool iFoundSSTs = false;
			    while( getline( is, is_line ) )
			    {
			          stringstream is_stream2( is_line );
				  // LSTs: (12 bit dynamical range)
				  is_stream2 >> iTemp;
                                  cout << iTemp << endl;
				  if( iTemp == "LST:" && !iFoundLSTs )
				  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 20. << "    " << 12 << "    " << 1 << endl;
				       }
                                       iFoundLSTs = true;
				  }
				  // FlashCam 
				  else if( iMST == "F" && ( iTemp == "MST-FlashCam:" || iTemp == "MST:" ) && !iFoundMSts )
				  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 20. << endl;
				       }
                                       iFoundMSts = true;
                                  }
				  // NectarCam (12 bit dynamical range)
				  else if ( iMST == "N" && ( iTemp == "MST-NectarCam:" || iTemp == "MST:" ) && !iFoundMSts )
				  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 20. << "    " << 12 << "    " << 1 << endl;
				       }
                                       iFoundMSts = true;
                                  }
				  // SSTs (with Astri)
				  else if(    ( iSST == "G" && iTemp == "GCT:" )
				           || ( iSST == "D" && iTemp == "SST-1M:" ) )
				  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 20. << endl;
				       }
                                  }
				  // SST (Astri)
				  else if( iSST == "A" && iTemp == "ASTRI:" )
				  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 20. << "    " << 12 << "    " << 1 << endl;
				       }
                                  }
				  // SCTs (15 deg FOV)
				  else if( iTemp == "SCT:" )
				  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 15. << endl;
				       }
				  }
                                  // SSTs
                                  else if( iTemp == "SST:" && !iFoundSSTs )
                                  {
				       while( !is_stream2.eof() )
				       {
					  is_stream2 >> iTemp2;
					  os << iTemp2 << "    " << 20. << endl;
				       }
                                       iFoundSSTs = true;
                                  }
				  if( iTemp == "SST-1M:" )
				  {
				       break;
                                  }
                            }
		       }
                    }
	        }
        }
	is.close();
	os.close();
}

/* 

   write all prod3 array layouts

*/

void writeAllArrays( bool iSouth = true )
{
      string iArrayList;
      if( iSouth )
      {
          iArrayList = "Prod3-Layouts-new2.lis";
      }
      else
      {
          iArrayList = "Prod3-Layouts-North.20151218.lis";
      }
      vector< string > ivMST;
      ivMST.push_back( "N" );
      ivMST.push_back( "F" );
      vector< string > ivSST;
      ivSST.push_back( "A" );
      ivSST.push_back( "G" );
      ivSST.push_back( "D" );

      // layouts
      vector< string > iA;
      if( iSouth )
      {
            iA.push_back( "3HB1" );
            iA.push_back( "3HB2" );
            iA.push_back( "3HB3" );
            iA.push_back( "3HB4" );

            iA.push_back( "3HS1" );

            iA.push_back( "3HD1" );
            iA.push_back( "3HD2" );

            iA.push_back( "3HI1" );

            iA.push_back( "3HT1" );

            iA.push_back( "3SS2" );
            iA.push_back( "3SS3" );
            iA.push_back( "3SD1" );
            iA.push_back( "3SD2" );
            iA.push_back( "3SD3" );
            iA.push_back( "3ST1" ); 
     }
     else
     {
            iA.push_back( "3AL4M15-1" );
            iA.push_back( "3AL4M15-2" );
            iA.push_back( "3AL4M15-3" );
            iA.push_back( "3AL4M15-4" );
            iA.push_back( "3AL4M15-5" );

            iA.push_back( "3AL3M15-1" );
            iA.push_back( "3AL3M15-2" );
            iA.push_back( "3AL3M15-3" );
            iA.push_back( "3AL3M15-4" );
            iA.push_back( "3AL3M15-5" );

            iA.push_back( "3AL2M15-1" );
            iA.push_back( "3AL2M15-2" );
            iA.push_back( "3AL2M15-3" );
            iA.push_back( "3AL2M15-4" );
            iA.push_back( "3AL2M15-5" );

            iA.push_back( "3BL4M13-1" );
            iA.push_back( "3BL4M13-2" );
            iA.push_back( "3BL4M13-3" );
            iA.push_back( "3BL4M13-4" );
            iA.push_back( "3BL4M13-5" );

            iA.push_back( "3BL3M13-1" );
            iA.push_back( "3BL3M13-2" );
            iA.push_back( "3BL3M13-3" );
            iA.push_back( "3BL3M13-4" );
            iA.push_back( "3BL3M13-5" );

            iA.push_back( "3BL2M13-1" );
            iA.push_back( "3BL2M13-2" );
            iA.push_back( "3BL2M13-3" );
            iA.push_back( "3BL2M13-4" );
            iA.push_back( "3BL2M13-5" );

            iA.push_back( "3CL4M09-1" );
            iA.push_back( "3CL4M09-2" );
            iA.push_back( "3CL4M09-3" );
            iA.push_back( "3CL4M09-4" );
            iA.push_back( "3CL4M09-5" );

            iA.push_back( "3CL3M09-1" );
            iA.push_back( "3CL3M09-2" );
            iA.push_back( "3CL3M09-3" );
            iA.push_back( "3CL3M09-4" );
            iA.push_back( "3CL3M09-5" );

            iA.push_back( "3CL2M09-1" );
            iA.push_back( "3CL2M09-2" );
            iA.push_back( "3CL2M09-3" );
            iA.push_back( "3CL2M09-4" );
            iA.push_back( "3CL2M09-5" );

            iA.push_back( "3DL4M05-1" );
            iA.push_back( "3DL4M05-2" );
            iA.push_back( "3DL4M05-3" );
            iA.push_back( "3DL4M05-4" );
            iA.push_back( "3DL4M05-5" );

            iA.push_back( "3DL3M05-1" );
            iA.push_back( "3DL3M05-2" );
            iA.push_back( "3DL3M05-3" );
            iA.push_back( "3DL3M05-4" );
            iA.push_back( "3DL3M05-5" ); 

            iA.push_back( "3DL2M05-1" );
            iA.push_back( "3DL2M05-2" );
            iA.push_back( "3DL2M05-3" );
            iA.push_back( "3DL2M05-4" );
            iA.push_back( "3DL2M05-5" ); 

            iA.push_back( "3ZL4M15-1" );
            iA.push_back( "3ZL4M15-2" ); 
            iA.push_back( "3ZL4M15-3" );
            iA.push_back( "3ZL4M15-4" );
            iA.push_back( "3ZL4M15-5" ); 

            iA.push_back( "3ZL3M15-1" );
            iA.push_back( "3ZL3M15-2" ); 
            iA.push_back( "3ZL3M15-3" );
            iA.push_back( "3ZL3M15-4" );
            iA.push_back( "3ZL3M15-5" ); 

            iA.push_back( "3ZL2M15-1" );
            iA.push_back( "3ZL2M15-2" ); 
            iA.push_back( "3ZL2M15-3" );
            iA.push_back( "3ZL2M15-4" );
            iA.push_back( "3ZL2M15-5" ); 
     }


      for( unsigned int a = 0; a < iA.size(); a++ )
      {
           for( unsigned int m = 0; m < ivMST.size(); m++ )
	   {
	        for( unsigned int s = 0; s < ivSST.size(); s++ )
		{
		     writeProd3_array( iArrayList, iA[a], ivMST[m], ivSST[s], iSouth );
                }
           }
      }
}
