#!/bin/bash
#
# script to analyse CTA MC files with lookup tables
#
# Author: Gernot Maier
#

TABFIL=TABLEFILE
RECID=RECONSTRUCTIONID
IFIL="IIIIFIL"
TFIL=TTTTFIL
ARRAY=ARRAYYY
DSET=DATASET
ADIR=AAAAADIR

# set the right observatory (environmental variables)
source $EVNDISPSYS/setObservatory.sh CTA

# output data and log files are written to this directory
ODIR=$CTA_USER_DATA_DIR"/analysis/AnalysisData/"$DSET"/"$ARRAY"/$ADIR/"
mkdir -p $ODIR

# delete old log files
rm -f $ODIR/$TFIL.log

# cp table file to temp disk
cp -v $CTA_EVNDISP_AUX_DIR/Tables/$TABFIL-$ARRAY.root $TMPDIR/$TABFIL-$ARRAY.root

# cp all evndisp root files to TMPDIR
NFIL=`cat $IFIL`
DDIR=$TMPDIR/data
mkdir -p $DDIR
for F in $NFIL
do
   cp $F $DDIR/
done
ls -1 $DDIR/*.root > $TMPDIR/iList.list

# check disk space on TMPDIR
du -h -c $TMPDIR

###############################################
# mscw_energy command line options
###############################################
MOPT="-pe -arrayrecid=$RECID -noNoTrigger -writeReconstructedEventsOnly -shorttree -useMedian=2 -add_mc_spectral_index=2.5"

# analyse MC file
# $EVNDISPSYS/bin/mscw_energy $MOPT -tablefile $TABFIL-$ARRAY.root -inputfilelist $IFIL -outputfile $ODIR/$TFIL.root >& $ODIR/$TFIL.log
$EVNDISPSYS/bin/mscw_energy $MOPT -tablefile $TMPDIR/$TABFIL-$ARRAY.root -inputfilelist $TMPDIR/iList.list -outputfile $TMPDIR/$TFIL.root >& $ODIR/$TFIL.log

mv -f -v $TMPDIR/$TFIL.root $ODIR/

# sleep
sleep 2

exit
