#!/bin/bash
#
# make tables for CTA
#
#
# Author: Gernot Maier
#

TFIL=TABLEFILE
RECID=RECONSTRUCTIONID
ARRAY=ARRRRRRR
DDIR="DATADIRECT"
CONE="CCC"
DSET="DATASET"

# set the right observatory (environmental variables)
source $EVNDISPSYS/setObservatory.sh CTA

# output data files are written to this directory
ODIR=$CTA_USER_DATA_DIR"/analysis/AnalysisData/$DSET/"$ARRAY"/Tables/"
mkdir -p $ODIR

# output log files are written to this directory
LDIR=$CTA_USER_DATA_DIR"/analysis/AnalysisData/$DSET/"$ARRAY"/Tables/"
mkdir -p $LDIR

# rename on-axis tables
if [ $CONE == "FALSE" ]
then
   TFIL=${TFIL}-onAxis
fi

# delete old log files
rm -f $LDIR/$TFIL-$ARRAY.log
# delete old table file (mscw_energy would otherwise stop with an error message)
rm -f $LDIR/$TFIL-$ARRAY.root

################################
# generate input file lists
DLIST=$TMPDIR/${DSET}.${ARRAY}.list
rm -f $DLIST
touch $DLIST
# for diffuse gamma rays: use also on-axis gamma rays to get sufficient statistics at high energies
if [ $CONE == "TRUE" ]
then
   find $CTA_USER_DATA_DIR/analysis/AnalysisData/$DSET/$ARRAY/gamma_cone/ -name "[1-9]*.root" >> $DLIST
#   find $CTA_USER_DATA_DIR/analysis/AnalysisData/$DSET/$ARRAY/gamma_onSource/ -name "[1-9]*.root" >> $DLIST
# on-axis gamma rays
else
   find $CTA_USER_DATA_DIR/analysis/AnalysisData/$DSET/$ARRAY/gamma_onSource/ -name "[1-9]*.root" >> $DLIST
fi

################################
# options for table filling

if [ $CONE == "TRUE" ]
then
   SETOFF="-CTAoffAxisBins"
fi

MOPT="$SETOFF -pe -filltables=1 -ze=20. -noise=250 -woff=0.0 -maxCoreError=250 -minImages=2 -limitEnergyReconstruction -write1DHistograms"

#########################################
# fill tables
cd $EVNDISPSYS/bin/
# input file list
./mscw_energy $MOPT -arrayrecid=$RECID -tablefile "$ODIR/$TFIL-$ARRAY.root" -inputfilelist ${DLIST} > $LDIR/$TFIL-$ARRAY.log
#########################################

# sleep
sleep 2

exit
