#!/bin/bash
# script to analyse files with lookup tables

# set observatory environmental variables
source $EVNDISPSYS/setObservatory.sh VTS

# parameters replaced by parent script using sed
TABFILE=TABLEFILE
RECID=RECONSTRUCTIONID
ODIR=OUTPUTDIRECTORY
INFILE=EVNDISPFILE
USEMODEL3D=USEMODEL3DMETHOD

INDIR=`dirname $INFILE`
BFILE=`basename $INFILE .root`

# temporary (scratch) directory
if [[ -n $TMPDIR ]]; then
    TEMPDIR=$TMPDIR
else
    TEMPDIR="$VERITAS_USER_DATA_DIR/TMPDIR"
fi
mkdir -p $TEMPDIR

#################################
# run analysis

MSCWLOGFILE="$ODIR/$BFILE.mscw.log"
rm -f $MSCWLOGFILE
cp -f -v $INFILE $TEMPDIR

MSCWDATAFILE="$ODIR/$BFILE.mscw.root"

if [[ $USEMODEL3D == "1" ]]; then
    MODEL3D="-model3d"
    echo "using Model3D for direction and core values"
fi

# run mscw_energy
    $EVNDISPSYS/bin/mscw_energy -tablefile $TABFILE -noshorttree -arrayrecid=$RECID -inputfile $TEMPDIR/$BFILE.root $MODEL3D -writeReconstructedEventsOnly=1 &> $MSCWLOGFILE

    # move output file from scratch and clean up
    cp -f -v $TEMPDIR/$BFILE.mscw.root $MSCWDATAFILE
    rm -f $TEMPDIR/$BFILE.mscw.root
    rm -f $TEMPDIR/$BFILE.root
    
# write info to log
echo "RUN$BFILE MSCWLOG $MSCWLOGFILE"
echo "RUN$BFILE MSCWDATA $MSCWDATAFILE"

exit
