/*!  VCTARequirements

    CTA requirements (hardwired values)

*/

#include "VCTARequirements.h"

VCTARequirements::VCTARequirements()
{
	fSetOfRequirementID = 0;
	
	setRequirement();
}

void VCTARequirements::listRequirementSets()
{
	cout << "ID 0: South, 50h" << endl;
	cout << "ID 1: South, 5h" << endl;
	cout << "ID 2: South, 0.5h" << endl;
	cout << "ID 3: North, 50h" << endl;
	cout << "ID 4: North, 5h" << endl;
	cout << "ID 5: North, 0.5h" << endl;
	cout << endl;
	cout << "Values based on MAN-PO/121004, version 3.001, August 5, 2013" << endl;
}

bool VCTARequirements::setRequirement( int iRequirementID )
{
	fSetOfRequirementID = iRequirementID;
	
	////////////////////////////////////////////////////////////
	// requirements
	// from MAN-PO/121004, version 2.5, June 14, 2013
	
	// from Figure 4
	fReqAngularResolution = new TGraph( 18 );
	fReqAngularResolution->SetPoint( 0, log10( 0.024 ), 0.394 );
	fReqAngularResolution->SetPoint( 1, log10( 0.028 ), 0.317 );
	fReqAngularResolution->SetPoint( 2, log10( 0.041 ), 0.220 );
	fReqAngularResolution->SetPoint( 3, log10( 0.059 ), 0.171 );
	fReqAngularResolution->SetPoint( 4, log10( 0.102 ), 0.127 );
	fReqAngularResolution->SetPoint( 5, log10( 0.167 ), 0.104 );
	fReqAngularResolution->SetPoint( 6, log10( 0.370 ), 0.081 );
	fReqAngularResolution->SetPoint( 7, log10( 1.147 ), 0.060 );
	fReqAngularResolution->SetPoint( 8, log10( 1.674 ), 0.054 );
	fReqAngularResolution->SetPoint( 9, log10( 2.905 ), 0.048 );
	fReqAngularResolution->SetPoint( 10, log10( 6.192 ), 0.041 );
	fReqAngularResolution->SetPoint( 11, log10( 9.638 ), 0.038 );
	fReqAngularResolution->SetPoint( 12, log10( 16.568 ), 0.034 );
	fReqAngularResolution->SetPoint( 13, log10( 33.101 ), 0.030 );
	fReqAngularResolution->SetPoint( 14, log10( 51.349 ), 0.028 );
	fReqAngularResolution->SetPoint( 15, log10( 88.158 ), 0.026 );
	fReqAngularResolution->SetPoint( 16, log10( 122.198 ), 0.025 );
	fReqAngularResolution->SetPoint( 17, log10( 256.706 ), 0.023 );
	setGraphPlottingStyle( fReqAngularResolution, 2, 1, 20, 1, 0, 2 );
	
	fGoalAngularResolution = 0;
	
	// from Figure 5
	fReqEnergyResolution = new TGraph( 4 );
	fReqEnergyResolution->SetPoint( 0, log10( 0.03 ), 0.40 );
	fReqEnergyResolution->SetPoint( 1, log10( 0.10 ), 0.20 );
	fReqEnergyResolution->SetPoint( 2, log10( 1.00 ), 0.10 );
	fReqEnergyResolution->SetPoint( 3, log10( 10.00 ), 0.10 );
	setGraphPlottingStyle( fReqEnergyResolution, 2 );
	
	fGoalEnergyResolution = 0;
	
	////////////////////////////////////////////////////////////
	// sensitivity requirements
	
	// SOUTH 50 h
	if( fSetOfRequirementID == 0 )
	{
		// from JH (mail 2013/08/30)
		fReqDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fReqDifferentialSensitivity, 2, 1., 20, 1., 0, 2 );
		fGoalDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fGoalDifferentialSensitivity, 3, 1., 20, 1., 0, 2 );
		for( int i = 0; i < 80; i++ )
		{
			fReqDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_req50_E2erg_south(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
			fGoalDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_goal50_E2erg_south(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
		}
	}
	// SOUTH 5 h
	else if( fSetOfRequirementID == 1 )
	{
		fReqDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fReqDifferentialSensitivity, 2, 1., 20, 1., 0, 2 );
		fGoalDifferentialSensitivity = 0;
		for( int i = 0; i < 80; i++ )
		{
			fReqDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_req5_E2erg_south(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
		}
	}
	// SOUTH 0.5 h
	else if( fSetOfRequirementID == 2 )
	{
		fReqDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fReqDifferentialSensitivity, 2, 1., 20, 1., 0, 2 );
		fGoalDifferentialSensitivity = 0;
		for( int i = 0; i < 80; i++ )
		{
			fReqDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_req05_E2erg_south(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
		}
	}
	// NORTH 50 h
	else if( fSetOfRequirementID == 3 )
	{
		// from JH (mail 2013/08/30)
		fReqDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fReqDifferentialSensitivity, 2, 1., 20, 1., 0, 2 );
		fGoalDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fGoalDifferentialSensitivity, 3, 1., 20, 1., 0, 2 );
		for( int i = 0; i < 80; i++ )
		{
			fReqDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_req50_E2erg_north(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
			fGoalDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_goal50_E2erg_north(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
		}
	}
	// NORTH 5 h
	else if( fSetOfRequirementID == 4 )
	{
		fReqDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fReqDifferentialSensitivity, 2, 1., 20, 1., 0, 2 );
		fGoalDifferentialSensitivity = 0;
		for( int i = 0; i < 80; i++ )
		{
			fReqDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_req5_E2erg_north(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
		}
	}
	// NORTH 0.5 h
	else if( fSetOfRequirementID == 5 )
	{
		fReqDifferentialSensitivity = new TGraph( 1 );
		setGraphPlottingStyle( fReqDifferentialSensitivity, 2, 1., 20, 1., 0, 2 );
		fGoalDifferentialSensitivity = 0;
		for( int i = 0; i < 80; i++ )
		{
			fReqDifferentialSensitivity->SetPoint( i, -1.9 + 0.05 * i, VCTASensitivityRequirements::Flux_req05_E2erg_north(
					TMath::Power( 10., -1.9 + 0.05 * i ) ) );
		}
	}
	else
	{
		cout << "VCTARequirements::setRequirement error: invalid requirement set ID: allowed is 0" << endl;
		return false;
	}
	
	////////////////////////////////////////////////////////
	// effective areas
	
	// SOUTH
	if( fSetOfRequirementID == 0 || fSetOfRequirementID == 1 || fSetOfRequirementID == 2 )
	{
		fReqEnergyRange_min = 0.02;
		fReqEnergyRange_max = 300.;
		
		fReqEffectiveArea = 0;
		fGoalEffectiveArea = new TGraph( 5 );
		fGoalEffectiveArea->SetPoint( 0, log10( 0.03 ), 1.0e4 );
		fGoalEffectiveArea->SetPoint( 1, log10( 0.10 ), 6.0e4 );
		fGoalEffectiveArea->SetPoint( 2, log10( 1.00 ), 1.0e6 );
		fGoalEffectiveArea->SetPoint( 3, log10( 10.00 ), 4.0e6 );
		fGoalEffectiveArea->SetPoint( 4, log10( 100.00 ), 7.0e6 );
		setGraphPlottingStyle( fGoalEffectiveArea, 3 );
	}
	// NORTH
	else if( fSetOfRequirementID == 3 || fSetOfRequirementID == 4 || fSetOfRequirementID == 5 )
	{
		fReqEnergyRange_min = 0.02;
		fReqEnergyRange_max = 20.;
		
		fReqEffectiveArea = 0;
		fGoalEffectiveArea = new TGraph( 4 );
		fGoalEffectiveArea->SetPoint( 0, log10( 0.03 ), 1.0e4 );
		fGoalEffectiveArea->SetPoint( 1, log10( 0.10 ), 6.0e4 );
		fGoalEffectiveArea->SetPoint( 2, log10( 1.00 ), 0.5e6 );
		fGoalEffectiveArea->SetPoint( 3, log10( 10.00 ), 1.0e6 );
		setGraphPlottingStyle( fGoalEffectiveArea, 3 );
	}
	return true;
}


TGraph* VCTARequirements::plotRequirement_DifferentialSensitivity( TCanvas* c, bool iPlotGoalRequirements )
{
	if( !c )
	{
		return 0;
	}
	
	if( !fReqDifferentialSensitivity )
	{
		cout << "VCTARequirements::plotRequirement_DifferentialSensitivity: error, no requirement given" << endl;
		return 0;
	}
	
	c->cd();
	plotRequirements( fReqDifferentialSensitivity, true, true );
	if( iPlotGoalRequirements )
	{
		plotRequirements( fGoalDifferentialSensitivity, true, true );
	}
	
	return fReqDifferentialSensitivity;
}

TGraph* VCTARequirements::plotRequirement_EnergyResolution( TCanvas* c, bool iPlotGoalRequirements )
{
	if( !c )
	{
		return 0;
	}
	
	if( !fReqEnergyResolution )
	{
		cout << "VCTARequirements::plotRequirement_EnergyResolution: error, no requirement given" << endl;
		return 0;
	}
	
	c->cd();
	plotRequirements( fReqEnergyResolution );
	if( iPlotGoalRequirements )
	{
		plotRequirements( fGoalEnergyResolution );
	}
	
	return fReqEnergyResolution;
}

TGraph* VCTARequirements::plotRequirement_AngularResolution( TCanvas* c, bool iPlotGoalRequirements )
{
	if( !c )
	{
		return 0;
	}
	
	if( !fReqAngularResolution )
	{
		cout << "VCTARequirements::plotRequirement_AngularResolution: error, no requirement given" << endl;
		return 0;
	}
	
	c->cd();
	plotRequirements( fReqAngularResolution, false, true );
	if( iPlotGoalRequirements )
	{
		plotRequirements( fGoalAngularResolution );
	}
	
	return fReqAngularResolution;
}

TGraph* VCTARequirements::plotRequirement_EffectiveArea( TCanvas* c, bool iPlotGoalRequirements )
{
	if( !c )
	{
		return 0;
	}
	
	if( !fReqEffectiveArea && !fGoalEffectiveArea )
	{
		cout << "VCTARequirements::plotRequirement_EffectiveArea: error, no requirement given" << endl;
		return 0;
	}
	
	c->cd();
	plotRequirements( fReqEffectiveArea, true );
	if( iPlotGoalRequirements )
	{
		plotRequirements( fGoalEffectiveArea, true );
	}
	
	return fReqAngularResolution;
}

void VCTARequirements::plotRequirements( TGraph* g, bool iLog, bool iLine )
{
	if( !g )
	{
		return;
	}
	
	if( iLine )
	{
		g->Draw( "l" );
		return;
	}
	
	double x = 0;
	double y = 0;
	double y_low = 0.;
	
	for( int i = 0; i < g->GetN(); i++ )
	{
		g->GetPoint( i, x, y );
		
		if( iLog )
		{
			y_low = 0.5 * y;
		}
		else
		{
			y_low = 0.8 * y;
		}
		TArrow* a = new TArrow( x, y, x, y_low, 0.01, "|-|>" );
		setArrowPlottingStyle( a, g->GetMarkerColor(), 1, 2 );
		a->Draw();
	}
}

/*

    field of view requirements: A-PERF-0010, A-PERF-0020

*/
double VCTARequirements::getFOVRequirement( double iE_lin_TeV )
{
	if( iE_lin_TeV < 0.1 )
	{
		return 2.5;
	}
	
	// 0.1 - 300 TeV
	
	return 3.;
}
