/*  \class VImageCleaningRunParameter

     data class for parameters for image cleaning

*/

#include "VImageCleaningRunParameter.h"

VImageCleaningRunParameter::VImageCleaningRunParameter( string iName )
{
        fName  = iName;
	fTelID = 0;
	
	fUseFixedThresholds = false;
	fImageCleaningMethod = 0;
	
	// default two-level threshold cleaning
	fimagethresh = 5.0;
	fborderthresh = 2.5;
	fbrightnonimagetresh = 2.5;
	
	// time cluster cleaning
	ftimecutpixel = 0.5;   
	ftimecutcluster = 2.0; 
	fminpixelcluster = 3;  
	floops = 2;            

	//cluster cleaning
	fnmaxcluster = 1;
	fminsizecluster = 0;
	
	// Trace Correlation Cleaning
	fCorrelationCleanBoardThresh = 1.0;   // S/N ratio of 1
	fCorrelationCleanCorrelThresh = 0.75; // Sample correlation coefficient of 0.75
	fCorrelationCleanNpixThresh = 15;     // Images whose number of pixels is above this value will skip correlation cleaning
	
	// time two-level cleaning
	ftimediff = 1.0;
}

bool VImageCleaningRunParameter::initialize()
{
	return true;
}

void VImageCleaningRunParameter::print()
{
	cout << "\t cleaning method \t \t" << getImageCleaningMethod() << " (" << getImageCleaningMethodIndex();
        cout << ", " << fName << ")" << endl;
        if( getImageCleaningMethod() != "TIMENEXTNEIGHBOUR" )
        {
            cout << "\t\t image/border\t\t\t" << fimagethresh << "/" << fborderthresh;
        }
	if( fUseFixedThresholds )
	{
		cout << "\t\t fixed cleaning thresholds" << endl;
	}
	else
	{
		cout << " (signal-to-noise cleaning thresholds)" << endl;
	}
	if( getImageCleaningMethodIndex() == 1 )
	{
		cout << "\t\t Tpixel/Tcluster/nMin/nLoops \t" << ftimecutpixel << "/" << ftimecutcluster
			 << "/" << fminpixelcluster << "/" << floops << endl;
	}
	if( getImageCleaningMethodIndex() == 3 )
	{
		cout << "\t\t using trace correlation cleaning: " << fCorrelationCleanBoardThresh << "/";
		cout << fCorrelationCleanCorrelThresh << "/" << fCorrelationCleanNpixThresh << "\t";
		cout << "BorderThresh/CorrelationThresh/MaxPixThresh" << endl;
	}
	if( getImageCleaningMethodIndex() == 4 )
	{
		cout << "\t\t time constraint between next neighbor pixels (samples): " << ftimediff << endl;
	}
	if( getImageCleaningMethodIndex() == 5 )
	{
		cout << "\t\t cluster cleaning; " ;
		if ( fnmaxcluster > 0) cout << "keep at most " << fnmaxcluster << " clusters; " ;
		if ( fminsizecluster > 0) cout << " min cluster size: " << fminsizecluster;
		if ( fminpixelcluster > 0) cout << " min no. pixels: " << fminpixelcluster;
		cout << endl;
	}
}


string VImageCleaningRunParameter::getImageCleaningMethod()
{
	if( fImageCleaningMethod == 1 )
	{
		return "TIMECLUSTERCLEANING";
	}
	else if( fImageCleaningMethod == 2 )
	{
		return "TIMENEXTNEIGHBOUR";
	}
	else if( fImageCleaningMethod == 3 )
	{
		return "TWOLEVELANDCORRELATION";
	}
	else if( fImageCleaningMethod == 4 )
	{
		return "TIMETWOLEVEL";
	}
	else if( fImageCleaningMethod == 5 )
	{
		return "CLUSTERCLEANING";
	}
	return "TWOLEVELCLEANING";
}

bool VImageCleaningRunParameter::setImageCleaningMethod( string iMethod )
{
	if( iMethod == "TWOLEVELCLEANING" )
	{
		fImageCleaningMethod = 0;
	}
	else if( iMethod == "TIMECLUSTERCLEANING" )
	{
		fImageCleaningMethod = 1;
	}
	else if( iMethod == "TIMENEXTNEIGHBOUR" )
	{
		fImageCleaningMethod = 2;
	}
	else if( iMethod == "TWOLEVELANDCORRELATION" )
	{
		fImageCleaningMethod = 3;
	}
	else if( iMethod == "TIMETWOLEVEL" )
	{
		fImageCleaningMethod = 4;
	}
	else if( iMethod == "CLUSTERCLEANING" )
	{
		fImageCleaningMethod = 5;
	}
	else
	{
		return false;
	}
	
	return true;
}
