
#include <string>
#include <fstream>
#include <iostream>
#include <cstdlib>
#include <algorithm>
#include <cmath>
#include <functional>


#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1F.h"
#include "TF1.h"
#include "TH2F.h"
#include "TMath.h"
#include "TProfile2D.h"

//For the Reffiles
Float_t Sel3D;		// elevation of shower direction (deg)
Float_t sigmaL3D;	// longitudinal (3D-length)
Float_t sigmaT3D; 	// transverse (3D-width)
Float_t Nc3D;     	// total number of Cherenkov photons emitted by the shower 
Float_t RWidth3D;
Float_t Saz3D;		// azimuth of shower direction (deg)
Float_t Smax3D;		// height of shower maximum (along the shower axis)
Float_t Depth3D;
Float_t Xcore3D;	// shower core in ground coordinates
Float_t Ycore3D;	// shower core in ground coordinates
Float_t Goodness3D;	// Goodness i.e how much a shower is gamma like
Float_t Xoff3D;		// Xcore offset 3D
Float_t Yoff3D;		// Ycore offset 3D
Float_t Energy3D;	// Energy3D
Float_t MCe0;
Float_t Chi2;
Float_t Goodbadugly;

Float_t OutGOODBADUGLY;
Float_t OutEnergy3D;	// Energy3D
Float_t OutMCe0;
Float_t OutSel3D;	// elevation of shower direction (deg)
Float_t OutsigmaL3D;	// longitudinal (3D-length)
Float_t OutsigmaT3D; 	// transverse (3D-width)
Float_t OutNc3D;     	// total number of Cherenkov photons emitted by the shower 
Float_t OutRWidth3D;
Float_t OutSaz3D;	// azimuth of shower direction (deg)
Float_t OutSmax3D;	// height of shower maximum (along the shower axis)
Float_t OutDepth3D;
Float_t OutXcore3D;	// shower core in ground coordinates
Float_t OutYcore3D;	// shower core in ground coordinates
Float_t OutGoodness3D;	// Goodness i.e how much a shower is gamma like
Float_t OutXoff3D;	// Xcore offset 3D
Float_t OutYoff3D;	
Float_t OutChi2;
			// elevation of shower direction (deg)
Float_t OutSel3DErr;	// elevation of shower direction (deg)
Float_t OutsigmaL3DErr;	// longitudinal (3D-length)
Float_t OutsigmaT3DErr; // transverse (3D-width)
Float_t OutNc3DErr;     // total number of Cherenkov photons emitted by the shower 
Float_t OutRWidth3DErr;
Float_t OutSaz3DErr;	// azimuth of shower direction (deg)
Float_t OutSmax3DErr;	// height of shower maximum (along the shower axis)
Float_t OutDepth3DErr;
Float_t OutXcore3DErr;
Float_t OutYcore3DErr;
Float_t OutBias;
Float_t OutEDiff;
Float_t OutNcBias;

Float_t OutRefSel3DErr;		// elevation of shower direction (deg)
Float_t OutRefsigmaL3DErr;	// longitudinal (3D-length)
Float_t OutRefsigmaT3DErr; 	// transverse (3D-width)
Float_t OutRefNc3DErr;     	// total number of Cherenkov photons emitted by the shower 
Float_t OutRefRWidth3DErr;
Float_t OutRefSaz3DErr;		// azimuth of shower direction (deg)
Float_t OutRefSmax3DErr;	// height of shower maximum (along the shower axis)
Float_t OutRefDepth3DErr;
Float_t OutRefXcore3DErr;
Float_t OutRefYcore3DErr;

Float_t Sel3DErr;	
Float_t sigmaL3DErr;	
Float_t sigmaT3DErr; 	
Float_t Nc3DErr;     	
Float_t RWidth3DErr;
Float_t Saz3DErr;	
Float_t Smax3DErr;	
Float_t Depth3DErr;
Float_t Xcore3DErr;
Float_t Ycore3DErr;

Float_t RefSel3DErr;	
Float_t RefsigmaL3DErr;	
Float_t RefsigmaT3DErr; 	
Float_t RefNc3DErr;     	
Float_t RefRWidth3DErr;
Float_t RefSaz3DErr;	
Float_t RefSmax3DErr;	
Float_t RefDepth3DErr;
Float_t RefXcore3DErr;
Float_t RefYcore3DErr;

Float_t Pros;
Float_t Pros0;
Float_t Pros1;
Float_t Pros2;
Float_t Pros3;
Float_t Pros4;
Float_t Pros5;
Float_t Pros6;
Float_t Pros7;
Float_t RMSEnergy3D;
Float_t DeltaRMSEnergy3D;

const char *NOICE[19] = {"50", "80", "120", "170", "230", "290", "370", "450", "075", "100", "150", "200", "250", "325", "425", "550", "750", "1000"};
int WinterSummer[2]={21, 22};
int TELVER[3]={4,5,6};
const char *Deg[10]={"00", "20", "30", "35", "40", "45", "50", "55", "60", "65"};	//shower angle, zenith angle 

using namespace std;

int main(int argc, char *argv[]){

	string datadir;
	int lowsim;
	int highsim;
	int lowtel;
	int hightel;
	int suwia;
	int suwib;
	int lownoise;
	int highnoise;
  	if (argc==10) {
   		datadir=argv[1];
		lowsim	 =argv[2];
		highsim	 =argv[3];
		lowtel	 =argv[4];
		hightel	 =argv[5];
		suwia	 =argv[6];
		suwib	 =argv[7];
		lownoise =argv[8];
		highnoise=argv[9];
  	}
	else {
    		cout <<"Usage: ./For_Plotting [datadir] [low simulation (CARE/GRISU)] [high simulation (CARE/GRISU)] [low telver (V4/V5/V6)] [high telver (V4/V5/V6)] [low summerwinter (21/22)] [high summerwinter (21/22)] [low noice (50-450 and 075-1000)] [high noice (50-450 and 075-1000)]"<<endl; //.txt list file for upgrade, wwoff assumed 0.5.
		cout <<"Example: ./For_Plotting $VERITASUSER_DATA_DIR/analysis/model3d/simulationanalysis 0 2 0 9 0 2 0 18" << endl;    
		cout <<"NOTE: if run locally, go in and change the naming of the inputfile according to your setup" << endl; 	
		cout <<"Note2: all cout comments have been removed to create a clean biascorrectionfile" << endl;	
		return 0;
  	}

	for ( int caregrisu = lowsim; caregrisu < highsim; caregrisu++){
		for ( int telev = lowtel; telv<hightel; telv++ ){
			for ( int iwintsum=suwia; iwintsum<suwib; iwintsum++){
				for ( int ideg=0; ideg<10; ideg+=1){		
					for ( int inoice=lownoise; inoice<highnoise; inoice++){
											
					char RunnumberFile[200];
					
					if (caregrisu == 0){	//CARE
						sprintf( RunnumberFile, "%s/9%d1200_ATM%s_ze%sdeg_offser0.5deg_NSB%sMHz.root", datadir, TELVER[ telev ], WinterSummer[ iwintsum ], Deg[ ideg ], NOICE[ inoice ]);
					}
					else {	//GRISU
						sprintf( RunnumberFile, "%s/9%d6500_ATM%s_ze%sdeg_offser0.5deg_NSB%sMHz.root", datadir, TELVER[ telev ], WinterSummer[ iwintsum ], Deg[ ideg ], NOICE[ inoice ]);
					}
					//cout << "Opening " << RunnumberFile << endl;
	
					// Setting branchadresses //
					TFile *InTfile = new TFile( RunnumberFile );
		
					TTree *InTTrees = (TTree*)InTfile->Get( "showerpars" );
					TTree *InTTreem = (TTree*)InTfile->Get( "model3Dpars" );
					
					InTTreem->SetBranchAddress( "Sel3D", 		&Sel3D );	
					InTTreem->SetBranchAddress( "sigmaL3D", 	&sigmaL3D );
					InTTreem->SetBranchAddress( "sigmaT3D", 	&sigmaT3D );
					InTTreem->SetBranchAddress( "Nc3D", 		&Nc3D );
					InTTreem->SetBranchAddress( "RWidth3D", 	&RWidth3D );
					InTTreem->SetBranchAddress( "Saz3D",		&Saz3D ); 
					InTTreem->SetBranchAddress( "Smax3D", 		&Smax3D );
					InTTreem->SetBranchAddress( "Depth3D", 		&Depth3D );
					InTTreem->SetBranchAddress( "Xcore3D", 		&Xcore3D );
					InTTreem->SetBranchAddress( "Ycore3D", 		&Ycore3D );
					InTTreem->SetBranchAddress( "Xoff3D", 		&Xoff3D );
					InTTreem->SetBranchAddress( "Yoff3D", 		&Yoff3D );
					InTTreem->SetBranchAddress( "Goodness3D", 	&Goodness3D );

					InTTreem->SetBranchAddress( "energy3d",		&Energy3D );
					InTTreem->SetBranchAddress( "GOODBADUGLY",	&Goodbadugly);

					InTTreem->SetBranchAddress( "PROS_Total3d",	&Pros );	
					InTTreem->SetBranchAddress( "PROS_Sel3d", 	&Pros0 );
					InTTreem->SetBranchAddress( "PROS_sigL3d", 	&Pros1 );
					InTTreem->SetBranchAddress( "PROS_sigT3d", 	&Pros2 );
					InTTreem->SetBranchAddress( "PROS_Nc3d", 	&Pros3 );
					InTTreem->SetBranchAddress( "PROS_RWidth3d", 	&Pros4 );
					InTTreem->SetBranchAddress( "PROS_Smax3d", 	&Pros5 );
					InTTreem->SetBranchAddress( "PROS_Depth3d", 	&Pros6 );
					InTTrees->SetBranchAddress( "MCe0", 		&MCe0 );  
					InTTrees->SetBranchAddress( "Chi2", 		&Chi2 );

					InTTreem->SetBranchAddress( "ErrorSel3D", 	&Sel3DErr );
					InTTreem->SetBranchAddress( "ErrorsigmaL3D", 	&sigmaL3DErr );
					InTTreem->SetBranchAddress( "ErrorsigmaT3D", 	&sigmaT3DErr );
					InTTreem->SetBranchAddress( "ErrorNc3D", 	&Nc3DErr );
					InTTreem->SetBranchAddress( "ErrRWidth3D", 	&RWidth3DErr );
					InTTreem->SetBranchAddress( "ErrorSaz3D",	&Saz3DErr ); 
					InTTreem->SetBranchAddress( "ErrorSmax3D", 	&Smax3DErr );
					InTTreem->SetBranchAddress( "ErrorXcore3D", 	&Xcore3DErr );
					InTTreem->SetBranchAddress( "ErrorYcore3D", 	&Ycore3DErr );

					InTTreem->SetBranchAddress( "RefErr_Sel3d", 	&RefSel3DErr );
					InTTreem->SetBranchAddress( "RefErr_sigmaL3d", 	&RefsigmaL3DErr );
					InTTreem->SetBranchAddress( "RefErr_sigmaT3d", 	&RefsigmaT3DErr );
					InTTreem->SetBranchAddress( "RefErr_Nc3d", 	&RefNc3DErr );
					InTTreem->SetBranchAddress( "RefErr_RWidth3d", 	&RefRWidth3DErr );
					InTTreem->SetBranchAddress( "RefErr_Saz3d",	&RefSaz3DErr ); 
					InTTreem->SetBranchAddress( "RefErr_Smax3d", 	&RefSmax3DErr );
					InTTreem->SetBranchAddress( "RefErr_Xcore3d", 	&RefXcore3DErr );
					InTTreem->SetBranchAddress( "RefErr_Ycore3d", 	&RefYcore3DErr );


					int nentries = InTTrees->GetEntries();										
					//cout << "Indata is loaded and has " << nentries << " enrties. " << endl;

					// Setting a outputname //
					char postOut[ 30 ];			
					sprintf( postOut, "_PLOTS_NEW.root");
					string Workname = RunnumberFile; 
					Workname.erase ( Workname.end() - 5, Workname.end() ); 
					string Outputfilename = Workname + postOut; 
					char *Outname=new char[ Outputfilename.size() + 1 ];  
					Outname[ Outputfilename.size() ] = 0;
					memcpy( Outname,Outputfilename.c_str(),Outputfilename.size() ); 
					//cout << "Output name is: " << Outname << endl;
	
					// Setting branches //
					TFile *newfile =  new TFile(	Outname, 	"recreate");
					TTree *Pars = new TTree( 	"Pars", 	"Pars");

					Pars ->Branch( "energy3d", 	&OutEnergy3D, 	"energy3d/F");
					Pars ->Branch( "TrueEnergy",	&OutMCe0, 	"TrueEnergy/F");
					Pars ->Branch( "Sel3D", 	&OutSel3D, 	"Sel3D/F");
					Pars ->Branch( "Saz3D", 	&OutSaz3D, 	"Saz3D/F");
					Pars ->Branch( "sigmaL3D", 	&OutsigmaL3D,	"sigmaL3D/F");
					Pars ->Branch( "sigmaT3D", 	&OutsigmaT3D, 	"sigmaT3D/F");
					Pars ->Branch( "Smax3D", 	&OutSmax3D, 	"Smax3D/F");
					Pars ->Branch( "Nc3D", 		&OutNc3D, 	"Nc3D/F");
					Pars ->Branch( "Xcore3D", 	&OutXcore3D, 	"Xcore3D/F");
					Pars ->Branch( "Ycore3D", 	&OutYcore3D, 	"Ycore3D/F");
					Pars ->Branch( "RWidth3D", 	&OutRWidth3D, 	"RWidth3D/F");
					Pars ->Branch( "Depth3D", 	&OutDepth3D, 	"Depth3D/F");
					Pars ->Branch( "Goodness3D", 	&OutGoodness3D, "Goodness3D/F");
					Pars ->Branch( "GOODBADUGLY", 	&OutGOODBADUGLY,"GOODBADUGLY/F");

					Pars ->Branch( "PROS_Total3d", 	&Pros, 		"PROS_Total3d/F");
					Pars ->Branch( "PROS_Sel3d", 	&Pros0, 	"PROS_Sel3d/F");
					Pars ->Branch( "PROS_sigL3d", 	&Pros1, 	"PROS_sigL3d/F");
					Pars ->Branch( "PROS_sigT3d", 	&Pros2, 	"PROS_sigT3d/F");
					Pars ->Branch( "PROS_Nc3d", 	&Pros3, 	"PROS_Nc3d/F");
					Pars ->Branch( "PROS_RWidth3d", &Pros4, 	"PROS_RWidth3d/F");
					Pars ->Branch( "PROS_Smax3d", 	&Pros5, 	"PROS_Smax3d/F");
					Pars ->Branch( "PROS_Depth3d", 	&Pros6, 	"PROS_Depth3d/F");
					Pars ->Branch( "EnergyBias3d", 	&OutBias, 	"EnergyBias3d/F");
					Pars ->Branch( "EnergyDiff3d", 	&OutEDiff, 	"EnergyDiff3d/F");

					Pars ->Branch( "NcBias", 	&OutNcBias, 	"NcBias/F");
					Pars ->Branch( "Chi2",	 	&OutChi2, 	"Chi2/F");

					Pars ->Branch( "ErrorSel3D", 	&OutSel3DErr, 	 "ErrorSel3D/F");
					Pars ->Branch( "ErrorSaz3D", 	&OutSaz3DErr, 	 "ErrorSel3D/F");
					Pars ->Branch( "ErrorSmax3D", 	&OutSmax3DErr, 	 "ErrorSmax3D/F");
					Pars ->Branch( "ErrorsigmaL3D", &OutsigmaL3DErr, "ErrorsigmaL3D/F");
					Pars ->Branch( "ErrorsigmaT3D", &OutsigmaT3DErr, "ErrorsigmaT3D/F");
					Pars ->Branch( "ErrorRWidth3D", &OutRWidth3DErr, "ErrorRWidth3D/F");
					Pars ->Branch( "ErrorXcore3D", 	&OutXcore3DErr,  "ErrorXcore3D/F");
					Pars ->Branch( "ErrorYcore3D", 	&OutYcore3DErr,  "ErrorYcore3D/F");
					Pars ->Branch( "ErrorNc3D", 	&OutNc3DErr, 	 "ErrorNc3D/F");

					Pars ->Branch( "RefErr_Sel3d", 	  &OutRefSel3DErr,    "RefErr_Sel3d/F");
					Pars ->Branch( "RefErr_Saz3d", 	  &OutRefSaz3DErr,    "RefErr_Saz3d/F");
					Pars ->Branch( "RefErr_Smax3d",   &OutRefSmax3DErr,   "RefErr_Smax3d/F");
					Pars ->Branch( "RefErr_sigmaL3d", &OutRefsigmaL3DErr, "RefErr_sigmaL3d/F");
					Pars ->Branch( "RefErr_sigmaT3d", &OutRefsigmaT3DErr, "RefErr_sigmaT3d/F");
					Pars ->Branch( "RefErr_RWidth3d", &OutRefRWidth3DErr, "RefErr_RWidth3d/F");
					Pars ->Branch( "RefErr_Xcore3d",  &OutRefXcore3DErr,  "RefErr_Xcore3d/F");
					Pars ->Branch( "RefErr_Ycore3d",  &OutRefYcore3DErr,  "RefErr_Ycore3d/F");
					Pars ->Branch( "RefErr_Nc3d", 	  &OutRefNc3DErr,     "RefErr_Nc3d/F");

					// Setting up a pasic plot to get biascorrection later //
					double logFrom = 0.1;
					double logTo = 50;
					const int nLogBins = 50;
					vector<double> logBins;
					logBins.push_back(logFrom);
					for( int i = 0; i < nLogBins; ++i) {
						double val = exp(log(logBins[i])+(log(logTo)-log(logFrom))/nLogBins);
						logBins.push_back(val);
					}
					double lBins[nLogBins+1];
					for(unsigned int i = 0; i < logBins.size(); ++i){
						lBins[i] = logBins[i];
					}

					TH2F *E3DmEtdEtvEt =  new TH2F("E3DmEtdEtvEt", 	"x vs y", nLogBins, lBins, 100, -1, 10); 
					TCanvas *c1 = new TCanvas("(E3D-Et)/EtvEt");
					
					// Setting up all the possible parameter vectors //
					vector < Float_t > VEnergy3D;
					vector < Float_t > VEnergy;
					vector < Float_t > VSel3D;
					vector < Float_t > VSaz3D;
					vector < Float_t > VSmax3D;
					vector < Float_t > VsigmaL3D;
					vector < Float_t > VsigmaT3D;
					vector < Float_t > VNc3D;
					vector < Float_t > VXcore3D;
					vector < Float_t > VYcore3D;
					vector < Float_t > VRWidth3D;
					vector < Float_t > VDepth3D;
					vector < Float_t > VGoodness3D;
					vector < Float_t > VChi2;
	
					vector < Float_t > VErrorSel3D;
					vector < Float_t > VErrorSaz3D;
					vector < Float_t > VErrorSmax3D;
					vector < Float_t > VErrorsigmaL3D;
					vector < Float_t > VErrorsigmaT3D;
					vector < Float_t > VErrorNc3D;
					vector < Float_t > VErrorXcore3D;
					vector < Float_t > VErrorYcore3D;
					vector < Float_t > VErrorRWidth3D;
	
					vector < Float_t > VErrorRefSel3D;
					vector < Float_t > VErrorRefSaz3D;
					vector < Float_t > VErrorRefSmax3D;
					vector < Float_t > VErrorRefsigmaL3D;
					vector < Float_t > VErrorRefsigmaT3D;
					vector < Float_t > VErrorRefNc3D;
					vector < Float_t > VErrorRefXcore3D;
					vector < Float_t > VErrorRefYcore3D;
					vector < Float_t > VErrorRefRWidth3D;
					
					vector < Float_t > VPros3D;
					vector < Float_t > VProsSel3D;
					vector < Float_t > VProssigL3D;
					vector < Float_t > VProssigT3D;
					vector < Float_t > VProsNc3D;
					vector < Float_t > VProsRWidth3D;
					vector < Float_t > VProsSmax3D;
					vector < Float_t > VProsDepth3D;
	
					vector < Float_t > VGOODBADUGLY;
	
					// Lots of inputparameters in case of more detailed cuts are wanted //
					for ( int i_in=0; i_in< nentries; i_in++ ) {
						InTTrees->GetEntry(	i_in);
						InTTreem->GetEntry(	i_in);
						VEnergy3D.push_back(	Energy3D);
						VEnergy.push_back(	MCe0);
						VSel3D.push_back(	Sel3D);
						VSaz3D.push_back(	Saz3D);
						VSmax3D.push_back(	Smax3D);
						VsigmaL3D.push_back(	sigmaL3D);
						VsigmaT3D.push_back(	sigmaT3D);
						VNc3D.push_back(	Nc3D);
						VXcore3D.push_back(	Xcore3D);
						VYcore3D.push_back(	Ycore3D);
						VRWidth3D.push_back(	RWidth3D);
						VDepth3D.push_back(	Depth3D);
						VChi2.push_back(	Chi2);
	
						VGoodness3D.push_back(	Goodness3D);
	
						VErrorSel3D.push_back(	 Sel3DErr);
						VErrorSaz3D.push_back(	 Saz3DErr);
						VErrorSmax3D.push_back(	 Smax3DErr);
						VErrorsigmaL3D.push_back(sigmaL3DErr);
						VErrorsigmaT3D.push_back(sigmaT3DErr);
						VErrorNc3D.push_back(	 Nc3DErr);
						VErrorXcore3D.push_back( Xcore3DErr);
						VErrorYcore3D.push_back( Ycore3DErr);
						VErrorRWidth3D.push_back(RWidth3DErr);
						VPros3D.push_back(	 Pros);
						VProsSel3D.push_back(	 Pros0);
						VProssigL3D.push_back(	 Pros1);
						VProssigT3D.push_back(	 Pros2);
						VProsNc3D.push_back(	 Pros3);
						VProsRWidth3D.push_back( Pros4);
						VProsSmax3D.push_back(	 Pros5);
						VProsDepth3D.push_back(	 Pros6);
	
						VErrorRefSel3D.push_back(	RefSel3DErr);
						VErrorRefSaz3D.push_back(	RefSaz3DErr);
						VErrorRefSmax3D.push_back(	RefSmax3DErr);
						VErrorRefsigmaL3D.push_back(	RefsigmaL3DErr);
						VErrorRefsigmaT3D.push_back(	RefsigmaT3DErr);
						VErrorRefNc3D.push_back(	RefNc3DErr);
						VErrorRefXcore3D.push_back(	RefXcore3DErr);
						VErrorRefYcore3D.push_back(	RefYcore3DErr);
						VErrorRefRWidth3D.push_back(	RefRWidth3DErr);
							
						VGOODBADUGLY.push_back(	Goodbadugly);
						
					}
					
					// Cuts //
					for ( int i_in2=0; i_in2 < nentries; i_in2+=1 ) {
						if ( VEnergy3D[i_in2] > 0.0 && VGOODBADUGLY[i_in2] > 0.0 
							/* OTHER CUTS, put in here */) {
	
							OutBias = (VEnergy3D[i_in2]-VEnergy[i_in2])/VEnergy[i_in2];
							E3DmEtdEtvEt->Fill(  VEnergy[i_in2], OutBias);
	
							Pars->Fill();
							Pars->GetCurrentFile();
						}
					}
	
					// Getting the bias correction values  with one landau 0.1-0.5, and two second degre polinomias (0.5-5 and 5-50), chosen based on extencive testing and experience //
					c1->SetLogx();
					TH1D* residual = (TH1D*)E3DmEtdEtvEt->ProfileX("X", 0, 100, "i")->Clone("resid");
	
					residual->Fit("landau", "", "", 0.1, 0.50);
					float lan_one   = residual->GetFunction("landau")->GetParameter(0);
					float lan_two   = residual->GetFunction("landau")->GetParameter(1);
					float lan_three = residual->GetFunction("landau")->GetParameter(2);
					cout << ANOYED << "_1	" << lan_one << "	" << lan_two << "	" << lan_three <<endl;
	
					residual->Fit("pol2", "", "", 0.50, 5.);
					float pol_one   = residual->GetFunction("pol2")->GetParameter(0);
					float pol_two   = residual->GetFunction("pol2")->GetParameter(1);
					float pol_three = residual->GetFunction("pol2")->GetParameter(2);
					cout << ANOYED << "_2 	" << pol_one << "	" << pol_two << "	" << pol_three <<endl;
	
					residual->Fit("pol2", "", "", 5, 50.);
					float pol_en  = residual->GetFunction("pol2")->GetParameter(0);
					float pol_tva = residual->GetFunction("pol2")->GetParameter(1);
					float pol_tre = residual->GetFunction("pol2")->GetParameter(2);
					cout << ANOYED << "_3	" << pol_en << "	" << pol_tva << "	" << pol_tre <<endl;
							
					newfile->Write();
					newfile->Close();
					//cout << "Finishing " << Outname << endl;
					delete[] Outname;
					}			
					}	
			}
		}
	}
}


